@@ -27,7 +27,6 @@
 # OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-import math
 import warnings
 
 import torch
@@ -39,7 +38,14 @@ from compressai.entropy_models import EntropyBottleneck, GaussianConditional
 from compressai.layers import GDN, MaskedConv2d
 from compressai.registry import register_model
 
-from .utils import conv, deconv, update_registered_buffers
+from .base import (
+    SCALES_LEVELS,
+    SCALES_MAX,
+    SCALES_MIN,
+    CompressionModel,
+    get_scale_table,
+)
+from .utils import conv, deconv
 
 __all__ = [
     "CompressionModel",
@@ -55,69 +61,6 @@ __all__ = [
 ]
 
 
-class CompressionModel(nn.Module):
-    """Base class for constructing an auto-encoder with at least one entropy
-    bottleneck module.
-
-    Args:
-        entropy_bottleneck_channels (int): Number of channels of the entropy
-            bottleneck
-    """
-
-    def __init__(self, entropy_bottleneck_channels, init_weights=None):
-        super().__init__()
-        self.entropy_bottleneck = EntropyBottleneck(entropy_bottleneck_channels)
-
-        if init_weights is not None:
-            warnings.warn(
-                "init_weights was removed as it was never functional",
-                DeprecationWarning,
-            )
-
-    def aux_loss(self):
-        """Return the aggregated loss over the auxiliary entropy bottleneck
-        module(s).
-        """
-        aux_loss = sum(
-            m.loss() for m in self.modules() if isinstance(m, EntropyBottleneck)
-        )
-        return aux_loss
-
-    def forward(self, *args):
-        raise NotImplementedError()
-
-    def update(self, force=False):
-        """Updates the entropy bottleneck(s) CDF values.
-
-        Needs to be called once after training to be able to later perform the
-        evaluation with an actual entropy coder.
-
-        Args:
-            force (bool): overwrite previous values (default: False)
-
-        Returns:
-            updated (bool): True if one of the EntropyBottlenecks was updated.
-
-        """
-        updated = False
-        for m in self.children():
-            if not isinstance(m, EntropyBottleneck):
-                continue
-            rv = m.update(force=force)
-            updated |= rv
-        return updated
-
-    def load_state_dict(self, state_dict):
-        # Dynamically update the entropy bottleneck buffers related to the CDFs
-        update_registered_buffers(
-            self.entropy_bottleneck,
-            "entropy_bottleneck",
-            ["_quantized_cdf", "_offset", "_cdf_length"],
-            state_dict,
-        )
-        super().load_state_dict(state_dict)
-
-
 @register_model("bmshj2018-factorized")
 class FactorizedPrior(CompressionModel):
     r"""Factorized Prior model from J. Balle, D. Minnen, S. Singh, S.J. Hwang,
@@ -125,6 +68,30 @@ class FactorizedPrior(CompressionModel):
     <https://arxiv.org/abs/1802.01436>`_, Int Conf. on Learning Representations
     (ICLR), 2018.
 
+    .. code-block:: none
+
+                  ┌───┐    y
+            x ──►─┤g_a├──►─┐
+                  └───┘    │
+                           ▼
+                         ┌─┴─┐
+                         │ Q │
+                         └─┬─┘
+                           │
+                     y_hat ▼
+                           │
+                           ·
+                        EB :
+                           ·
+                           │
+                     y_hat ▼
+                           │
+                  ┌───┐    │
+        x_hat ──◄─┤g_s├────┘
+                  └───┘
+
+        EB = Entropy bottleneck
+
     Args:
         N (int): Number of channels
         M (int): Number of channels in the expansion layers (last layer of the
@@ -132,7 +99,9 @@ class FactorizedPrior(CompressionModel):
     """
 
     def __init__(self, N, M, **kwargs):
-        super().__init__(entropy_bottleneck_channels=M, **kwargs)
+        super().__init__(**kwargs)
+
+        self.entropy_bottleneck = EntropyBottleneck(M)
 
         self.g_a = nn.Sequential(
             conv(3, N),
@@ -200,7 +169,7 @@ class FactorizedPriorReLU(FactorizedPrior):
     N. Johnston: `"Variational Image Compression with a Scale Hyperprior"
     <https://arxiv.org/abs/1802.01436>`_, Int Conf. on Learning Representations
     (ICLR), 2018.
-    GDN activations are replaced by ReLU
+    GDN activations are replaced by ReLU.
 
     Args:
         N (int): Number of channels
@@ -209,7 +178,7 @@ class FactorizedPriorReLU(FactorizedPrior):
     """
 
     def __init__(self, N, M, **kwargs):
-        super().__init__(entropy_bottleneck_channels=M, **kwargs)
+        super().__init__(N=N, M=M, **kwargs)
 
         self.g_a = nn.Sequential(
             conv(3, N),
@@ -232,16 +201,6 @@ class FactorizedPriorReLU(FactorizedPrior):
         )
 
 
-# From Balle's tensorflow compression examples
-SCALES_MIN = 0.11
-SCALES_MAX = 256
-SCALES_LEVELS = 64
-
-
-def get_scale_table(min=SCALES_MIN, max=SCALES_MAX, levels=SCALES_LEVELS):
-    return torch.exp(torch.linspace(math.log(min), math.log(max), levels))
-
-
 @register_model("bmshj2018-hyperprior")
 class ScaleHyperprior(CompressionModel):
     r"""Scale Hyperprior model from J. Balle, D. Minnen, S. Singh, S.J. Hwang,
@@ -249,6 +208,31 @@ class ScaleHyperprior(CompressionModel):
     <https://arxiv.org/abs/1802.01436>`_ Int. Conf. on Learning Representations
     (ICLR), 2018.
 
+    .. code-block:: none
+
+                  ┌───┐    y     ┌───┐  z  ┌───┐ z_hat      z_hat ┌───┐
+            x ──►─┤g_a├──►─┬──►──┤h_a├──►──┤ Q ├───►───·⋯⋯·───►───┤h_s├─┐
+                  └───┘    │     └───┘     └───┘        EB        └───┘ │
+                           ▼                                            │
+                         ┌─┴─┐                                          │
+                         │ Q │                                          ▼
+                         └─┬─┘                                          │
+                           │                                            │
+                     y_hat ▼                                            │
+                           │                                            │
+                           ·                                            │
+                        GC : ◄─────────────────────◄────────────────────┘
+                           ·                 scales_hat
+                           │
+                     y_hat ▼
+                           │
+                  ┌───┐    │
+        x_hat ──◄─┤g_s├────┘
+                  └───┘
+
+        EB = Entropy bottleneck
+        GC = Gaussian conditional
+
     Args:
         N (int): Number of channels
         M (int): Number of channels in the expansion layers (last layer of the
@@ -256,7 +240,9 @@ class ScaleHyperprior(CompressionModel):
     """
 
     def __init__(self, N, M, **kwargs):
-        super().__init__(entropy_bottleneck_channels=N, **kwargs)
+        super().__init__(**kwargs)
+
+        self.entropy_bottleneck = EntropyBottleneck(N)
 
         self.g_a = nn.Sequential(
             conv(3, N),
@@ -316,15 +302,6 @@ class ScaleHyperprior(CompressionModel):
             "likelihoods": {"y": y_likelihoods, "z": z_likelihoods},
         }
 
-    def load_state_dict(self, state_dict):
-        update_registered_buffers(
-            self.gaussian_conditional,
-            "gaussian_conditional",
-            ["_quantized_cdf", "_offset", "_cdf_length", "scale_table"],
-            state_dict,
-        )
-        super().load_state_dict(state_dict)
-
     @classmethod
     def from_state_dict(cls, state_dict):
         """Return a new model instance from `state_dict`."""
@@ -334,13 +311,6 @@ class ScaleHyperprior(CompressionModel):
         net.load_state_dict(state_dict)
         return net
 
-    def update(self, scale_table=None, force=False):
-        if scale_table is None:
-            scale_table = get_scale_table()
-        updated = self.gaussian_conditional.update_scale_table(scale_table, force=force)
-        updated |= super().update(force=force)
-        return updated
-
     def compress(self, x):
         y = self.g_a(x)
         z = self.h_a(torch.abs(y))
@@ -370,6 +340,31 @@ class MeanScaleHyperprior(ScaleHyperprior):
     Priors for Learned Image Compression" <https://arxiv.org/abs/1809.02736>`_,
     Adv. in Neural Information Processing Systems 31 (NeurIPS 2018).
 
+    .. code-block:: none
+
+                  ┌───┐    y     ┌───┐  z  ┌───┐ z_hat      z_hat ┌───┐
+            x ──►─┤g_a├──►─┬──►──┤h_a├──►──┤ Q ├───►───·⋯⋯·───►───┤h_s├─┐
+                  └───┘    │     └───┘     └───┘        EB        └───┘ │
+                           ▼                                            │
+                         ┌─┴─┐                                          │
+                         │ Q │                                          ▼
+                         └─┬─┘                                          │
+                           │                                            │
+                     y_hat ▼                                            │
+                           │                                            │
+                           ·                                            │
+                        GC : ◄─────────────────────◄────────────────────┘
+                           ·                 scales_hat
+                           │                 means_hat
+                     y_hat ▼
+                           │
+                  ┌───┐    │
+        x_hat ──◄─┤g_s├────┘
+                  └───┘
+
+        EB = Entropy bottleneck
+        GC = Gaussian conditional
+
     Args:
         N (int): Number of channels
         M (int): Number of channels in the expansion layers (last layer of the
@@ -377,7 +372,7 @@ class MeanScaleHyperprior(ScaleHyperprior):
     """
 
     def __init__(self, N, M, **kwargs):
-        super().__init__(N, M, **kwargs)
+        super().__init__(N=N, M=M, **kwargs)
 
         self.h_a = nn.Sequential(
             conv(M, N, stride=1, kernel_size=3),
@@ -442,6 +437,35 @@ class JointAutoregressiveHierarchicalPriors(MeanScaleHyperprior):
     Priors for Learned Image Compression" <https://arxiv.org/abs/1809.02736>`_,
     Adv. in Neural Information Processing Systems 31 (NeurIPS 2018).
 
+    .. code-block:: none
+
+                  ┌───┐    y     ┌───┐  z  ┌───┐ z_hat      z_hat ┌───┐
+            x ──►─┤g_a├──►─┬──►──┤h_a├──►──┤ Q ├───►───·⋯⋯·───►───┤h_s├─┐
+                  └───┘    │     └───┘     └───┘        EB        └───┘ │
+                           ▼                                            │
+                         ┌─┴─┐                                          │
+                         │ Q │                                   params ▼
+                         └─┬─┘                                          │
+                     y_hat ▼                  ┌─────┐                   │
+                           ├──────────►───────┤  CP ├────────►──────────┤
+                           │                  └─────┘                   │
+                           ▼                                            ▼
+                           │                                            │
+                           ·                  ┌─────┐                   │
+                        GC : ◄────────◄───────┤  EP ├────────◄──────────┘
+                           ·     scales_hat   └─────┘
+                           │      means_hat
+                     y_hat ▼
+                           │
+                  ┌───┐    │
+        x_hat ──◄─┤g_s├────┘
+                  └───┘
+
+        EB = Entropy bottleneck
+        GC = Gaussian conditional
+        EP = Entropy parameters network
+        CP = Context prediction (masked convolution)
+
     Args:
         N (int): Number of channels
         M (int): Number of channels in the expansion layers (last layer of the
