@@ -32,6 +32,37 @@ import torch
 from torch import Tensor
 
 
+def compute_padding(in_h: int, in_w: int, *, out_h=None, out_w=None, min_div=1):
+    """Returns tuples for padding and unpadding.
+
+    Args:
+        in_h: Input height.
+        in_w: Input width.
+        out_h: Output height.
+        out_w: Output width.
+        min_div: Length that output dimensions should be divisible by.
+    """
+    if out_h is None:
+        out_h = (in_h + min_div - 1) // min_div * min_div
+    if out_w is None:
+        out_w = (in_w + min_div - 1) // min_div * min_div
+
+    if out_h % min_div != 0 or out_w % min_div != 0:
+        raise ValueError(
+            f"Padded output height and width are not divisible by min_div={min_div}."
+        )
+
+    left = (out_w - in_w) // 2
+    right = out_w - in_w - left
+    top = (out_h - in_h) // 2
+    bottom = out_h - in_h - top
+
+    pad = (left, right, top, bottom)
+    unpad = (-left, -right, -top, -bottom)
+
+    return pad, unpad
+
+
 def quantize_ste(x: Tensor) -> Tensor:
     """
     Rounding with non-zero gradients. Gradients are approximated by replacing
