@@ -1,16 +1,31 @@
-# Copyright 2020 InterDigital Communications, Inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+# Copyright (c) 2021-2022, InterDigital Communications, Inc
+# All rights reserved.
+
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted (subject to the limitations in the disclaimer
+# below) provided that the following conditions are met:
+
+# * Redistributions of source code must retain the above copyright notice,
+#   this list of conditions and the following disclaimer.
+# * Redistributions in binary form must reproduce the above copyright notice,
+#   this list of conditions and the following disclaimer in the documentation
+#   and/or other materials provided with the distribution.
+# * Neither the name of InterDigital Communications, Inc nor the names of its
+#   contributors may be used to endorse or promote products derived from this
+#   software without specific prior written permission.
+
+# NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY
+# THIS LICENSE. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
+# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
+# NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 """
 Collect performance metrics of published traditional or end-to-end image
 codecs.
@@ -18,11 +33,11 @@ codecs.
 import argparse
 import json
 import multiprocessing as mp
-import os
 import sys
 
 from collections import defaultdict
 from itertools import starmap
+from pathlib import Path
 from typing import List
 
 from .codecs import AV1, BPG, HM, JPEG, JPEG2000, TFCI, VTM, Codec, WebP
@@ -52,19 +67,16 @@ def func(codec, i, *args):
 def collect(
     codec: Codec,
     dataset: str,
-    qualities: List[int],
+    qps: List[int],
     metrics: List[str],
     num_jobs: int = 1,
 ):
-    if not os.path.isdir(dataset):
+    if not Path(dataset).is_dir():
         raise OSError(f"No such directory: {dataset}")
 
-    filepaths = [
-        os.path.join(dirpath, f)
-        for dirpath, _, filenames in os.walk(dataset)
-        for f in filenames
-        if os.path.splitext(f)[-1].lower() in IMG_EXTENSIONS
-    ]
+    filepaths = []
+    for ext in IMG_EXTENSIONS:
+        filepaths.extend(Path(dataset).rglob(f"*{ext}"))
 
     pool = mp.Pool(num_jobs) if num_jobs > 1 else None
 
@@ -73,7 +85,7 @@ def collect(
         sys.exit(1)
 
     args = [
-        (codec, i, f, q, metrics) for i, q in enumerate(qualities) for f in filepaths
+        (codec, i, f, q, metrics) for i, q in enumerate(qps) for f in sorted(filepaths)
     ]
 
     if pool:
@@ -81,7 +93,7 @@ def collect(
     else:
         rv = list(starmap(func, args))
 
-    results = [defaultdict(float) for _ in range(len(qualities))]
+    results = [defaultdict(float) for _ in range(len(qps))]
 
     for i, metrics in rv:
         for k, v in metrics.items():
@@ -120,13 +132,11 @@ def setup_common_args(parser):
     )
     parser.add_argument(
         "-q",
-        "--quality",
-        dest="qualities",
-        metavar="Q",
-        default=[75],
-        nargs="+",
-        type=int,
-        help="quality parameter (default: %(default)s)",
+        "--qps",
+        dest="qps",
+        type=str,
+        default="75",
+        help="list of quality/quantization parameter (default: %(default)s)",
     )
     parser.add_argument(
         "--metrics",
@@ -147,10 +157,11 @@ def main(argv):
 
     codec_cls = next(c for c in codecs if c.__name__.lower() == args.codec)
     codec = codec_cls(args)
+    qps = [int(q) for q in args.qps.split(",") if q]
     results = collect(
         codec,
         args.dataset,
-        args.qualities,
+        sorted(qps),
         args.metrics,
         args.num_jobs,
     )
